package org.greenstone.gsdl3.gs3build.doctypes;

import java.util.ArrayList;
import java.io.File;
import java.net.URL;
import org.w3c.dom.Element;

import org.greenstone.gsdl3.util.GSFile;
import org.greenstone.gsdl3.util.Processing;
import org.greenstone.gsdl3.gs3build.util.HTTPTools;

public class GS2PerlRecogniser extends AbstractRecogniser
{
    public String plugin_name;
    public GS2PerlRecogniser()
    { 
	//this.preferred_mime_type = "application/pdf";
	//this.preferred_mime_type = "text/plain";
	this.filename_extensions = new ArrayList();
	//this.filename_extensions.add(".pdf");
	//this.filename_extensions.add(".txt");
	this.document_type = METSDocument.METS_DOCUMENT_TYPE;
    }

    public boolean configure(Element config_elem) {
	String mime_type = config_elem.getAttribute("mimeType");
	if (mime_type.equals("")) {
	    System.err.println("GS2PerlRecogniser configure Error: no mimetype specified");
	    return false;
	}
	this.preferred_mime_type = mime_type;
	String plugin = config_elem.getAttribute("plugin");
	if (plugin.equals("")) {
	    System.err.println("GS2PerlRecogniser configure Error: no plugin specified");
	    return false;
	}
	this.plugin_name = plugin;
	
	String extensions = config_elem.getAttribute("filenameExtensions");
	if (extensions.equals("")) {
	    System.err.println("GS2PerlRecogniser configure Error: no filename extensions specified");
	    return false;
	}
	String [] exts = extensions.split("|");
	for (int i=0; i<exts.length; i++) {
	    this.filename_extensions.add(exts[i]);
	}
	return true;
	
    }
    public boolean parseDocument(URL url)
    { 
	String filename = null;
	
	if (url.getProtocol().equals("file")) {
	    filename = url.getPath();
	}
	
	if (filename != null) {
	    if ( isAcceptedFilename(filename)) {
		
		System.out.println("Posting "+document_type+" Document " + filename);
		METSDocument doc = processFile(url, filename);
		
		if (doc != null) {
		    this.list_repository.addDocument(doc);
		    return true;
		}
		return false;
	    }
	} else {
	    // Get Mime type remotely, and then proceed if required
	    String mimeType = HTTPTools.getMIMEType(url);
	    
	    if (mimeType == this.preferred_mime_type) {
		System.out.println("Posting "+document_type+" Document " + url.toString());
		METSDocument doc = processFile(url, null);
		if (doc != null) {
		
		    this.list_repository.addDocument(doc);
		    return true;
		}
		return false;
	    }
	}
	return false;
    }


    protected METSDocument processFile(URL url, String filename) {

	// we can only do files at this stage
	if (filename == null) return null;
	File file = new File(filename);
	String local_dir = file.getParent();
	System.err.println("local dir = "+local_dir);
	
	//String command = "importone.pl -saveas METS -collectdir /research/kjdon/home/gsdl3/web/sites/localsite/collect -importdir "+local_dir+" -archivedir "+local_dir+" -- kath TEXTPlug "+ file.getName();
	String command = "importone.pl -saveas METS -collectdir "+GSFile.collectDir(this.coll_manager.siteHome)+" -importdir "+local_dir+" -archivedir "+local_dir+" -- "+this.coll_manager.collectionName +" " +this.plugin_name+" "+ file.getName();
	System.err.println("Command = "+command);
	
	int exit_value = Processing.runProcess(command);
	if (exit_value == 0) {
	    System.out.println("converted success");
	} else {
	    System.out.println("Unable to convert to mets");
	    
	    return null ;
	}
	
	//read in the archives.inf file to see the directory for the new mets
	File arcinfo = new File(local_dir, "archives.inf");
	if (!arcinfo.isFile()) {
	    System.err.println("cant find archives.inf file ("+arcinfo.getPath()+", aborting");
	    return null;
	}
	try {
	String content = new String(GSFile.readFile(arcinfo.getPath()));
	content = content.trim();
	System.err.println("content =  "+content);
	String items[] = content.split("\\s+");
	String filepath = items[1];
	// delete the archive file
	arcinfo.delete();
	File mets_file = new File(local_dir, filepath);
	System.err.println("mets file = "+mets_file.getPath());
	return new METSDocument(mets_file.toURL());
	} catch (Exception e) {
	    System.err.println("processfile: "+e);
	    return null;
	}
    }
}

    

