package org.greenstone.gsdl3.gs3build.doctypes;

import java.io.*;

import java.net.URL;

import javax.xml.parsers.*;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.Text;

import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

import org.greenstone.gsdl3.gs3build.metadata.*;
import org.greenstone.gsdl3.gs3build.util.HTMLDoc;
import org.greenstone.gsdl3.gs3build.doctypes.DocumentInterface;
import org.greenstone.gsdl3.gs3build.xpointer.XPointer;

public class METSDocument extends AbstractDocument {

    public static final String METS_DOCUMENT_TYPE = "METS";
    Document domDocument;
    String original_location;
    
    public METSDocument(DocumentID id) {
    	super(id);
	//domDocument = null;
    }

    public METSDocument(URL url) {
	super(url);
	
	if (url.getProtocol().equals("file")) {
	    this._parseFile(new File(url.getPath()));
	} else {
	    System.err.println("Error: METSDocument must currently be a file");
	    System.err.println("       URL = " + url.getPath());
	}
    }
    
    private void _parseFile(File file) {
	
	String parseFilePath;
	try {
	    DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
	    DocumentBuilder builder = factory.newDocumentBuilder();
	    domDocument = builder.parse(file);

	    //int filePosition = file.getPath().indexOf("import/")+7;
	    //parseFilePath = file.getPath().substring(0, filePosition);
	    parseFilePath = file.getParent(); // all refs should be relative to the current doc, or absolute
	    parseFilePath += File.separator;
	    this.original_location = parseFilePath;
	    // TODO: get all the types in the tree
      
	    // TODO: do a traverse, and thus cope with elements-within-elements if needsbe, but
	    // this shouldn't happen except in a directly defined situation - actually doing the
	    // parsing in part inside each node would work well provided one checked for a node
	    // having already been done...
	    System.out.println("Read");
      
	    // Get METSDmdSec metadata sections
	    NodeList dmdSecs = domDocument.getElementsByTagName("mets:dmdSec");
	    
	    this.metadata = METSDescriptiveSet.parseXML(dmdSecs);

	    // Get METSFileSec sections
	    NodeList fileSecs = domDocument.getElementsByTagName("mets:fileSec");
	    this.fileSet = METSFileSet.parseXML(fileSecs, fileSet, parseFilePath);
	    // Get METSStructureMap Sections
	    NodeList structMapSecs = domDocument.getElementsByTagName("mets:structMap");
	    this.structureSet = METSStructureSet.parseXML(structMapSecs);

	    //for (int g = 0; g < fileSecs.getLength(); g ++) {
		//Schema schema = new Schema(schemas.item(s));
		//this._parseFileSec(fileSecs.item(g));
	    //}
      
	    // Get document metadata sections
	    //NodeList dmdSecs = document.getElementsByTagName("mets:dmdSec");
      
	    //Schema schema = new Schema(schemas.item(s));
	    //this.metadata = METSDescriptiveSet.parseXML(fileSecs);
	} catch (FactoryConfigurationError e) {
	    System.out.println(e);
	} catch (ParserConfigurationException ex) {
	    System.out.println(ex);
	} catch (SAXException ex) {
	    System.out.println(ex);
	} catch (IOException ex) {
	    System.out.println(ex);
	}
    }
    /*
    private void _parseFileSec(Node fileSec) {
	// this is in effect a group without a sense of 'self'...
	this._parseFileGroup((Element) fileSec, null);
    }

    private void _parseFileGroup(Element groupTag, METSFileGroup group) {
	NodeList children = groupTag.getChildNodes();

	for (int c = 0; c < children.getLength(); c ++) {
	    if (children.item(c).getNodeType() != org.w3c.dom.Node.ELEMENT_NODE) {
		continue;
	    }

	    System.out.println(children.item(c));
    
	    Element element = (Element) children.item(c);

	    if (element.getNodeName().equals("mets:File")) {
		if (group != null) {
		    METSFile file = METSFile.parseXML(element, group);
		} else {
		    // TODO: error
		}
	    } else if (element.getNodeName().equals("mets:fileGrp")) {
		// recurse
		METSFileGroup childGroup = new METSFileGroup(element.getAttribute("ID"));

		this._parseFileGroup(element, childGroup);
		if (group != null) {
		    group.addGroup(childGroup);
		} else { 
		    this.fileSet.addGroup(childGroup);
		}
	    }
	}
	}*/
	
    public String getDocumentType() {
	return METS_DOCUMENT_TYPE;
    }
	
    public String getDocumentText() {
	// TODO: make this more than a dummy function!
	HTMLDoc metsDoc;
	URL url = (URL) this.fileSet.getFile(0).getLocation();

	//this.getSectionText("1");
	
	if (url.getProtocol().equals("file")) {
	    String path = url.getPath();
	    File f = new File(path);
	    if (!f.isAbsolute() && this.original_location != null) {
		path = this.original_location + path;
		System.err.println("new path ="+path);
	    }
	    metsDoc = new HTMLDoc(url,path);
	} else {
	    metsDoc = new HTMLDoc(url);
	}
	return metsDoc.getContent();
    }
    public Document getDOMDocument() 
    {
	if (domDocument == null) {
	    URL url = (URL) this.fileSet.getFile(0).getLocation();
	    try {
		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
		DocumentBuilder builder = factory.newDocumentBuilder();
		domDocument = builder.parse(new File(url.getPath()));
	    } catch (FactoryConfigurationError e) {
		System.out.println(e);
	    } catch (ParserConfigurationException ex) {
		System.out.println(ex);
	    } catch (SAXException ex) {
		System.out.println(ex);
	    } catch (IOException ex) {
		System.out.println(ex);
	    }
	}
	return domDocument;
    }

    private XPointer getXPointer(METSDivision division)
    {
	String fileId = division.getDefaultFileReference();

	if (fileId == null) {
	    System.err.println("Unable to obtain file reference for section " + division.getLabel());
	    return null;
	}
	
	METSFileGroup fileGroup = this.getDocumentFiles().getGroup(fileId);
	if (fileGroup == null) {
	    System.err.println("Unable to obtain file reference for filegroup " + fileId);
	    return null;
	}
	METSFile file = fileGroup.getFile(0);
	if (file == null) {
	    System.err.println("Unable to obtain any files within filegroup " + fileId);
	    return null;
	}
	URL url = file.getLocation();
	XPointer xpointer = XPointer.processXPointer(domDocument, url);
	
	return xpointer;
    }
    
    private XPointer getXPointer(String sectionId)
    {
	if (domDocument == null) {
	    URL url = (URL) this.fileSet.getFile(0).getLocation();
	    try {
		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
		DocumentBuilder builder = factory.newDocumentBuilder();
		domDocument = builder.parse(new File(url.getPath()));
	    } catch (FactoryConfigurationError e) {
		System.out.println(e);
	    } catch (ParserConfigurationException ex) {
		System.out.println(ex);
	    } catch (SAXException ex) {
		System.out.println(ex);
	    } catch (IOException ex) {
		System.out.println(ex);       
	    }
	}
	
	METSDivision division = 
	    this.getDocumentStructure().getDivision(METSStructureSet.GSDL3_SECTION_STRUCTURE, sectionId);
	
	if(division == null) {
	    return null;
	}
      	return this.getXPointer(division);
    }

    public Node getSectionStartNode(METSDivision division)
    {
	XPointer xpointer = this.getXPointer(division);
	return xpointer.getStartNode();
    }

    public Node getSectionStartNode(String sectionId)
    {
	XPointer xpointer = this.getXPointer(sectionId);
	return xpointer.getStartNode();
    }
    
    public String getSectionText(String sectionId) 
    {
	XPointer xpointer = this.getXPointer(sectionId);
	
	if (xpointer == null) {
	    return "";
	}

	//get the XML content of the xpointer
	return xpointer.toString();
    }
}


