package org.greenstone.gsdl3.gs3build.indexers;

import org.greenstone.mgpp.MGPPPassesWrapper;

import org.greenstone.gsdl3.gs3build.doctypes.DocumentID;
import org.greenstone.gsdl3.gs3build.doctypes.DocumentInterface;
import org.greenstone.gsdl3.gs3build.doctypes.HTMLDocument;
import org.greenstone.gsdl3.gs3build.doctypes.METSDocument;
import org.greenstone.gsdl3.util.Misc;
import org.greenstone.gsdl3.util.GSFile;
import org.greenstone.gsdl3.util.GSXML;
import org.greenstone.gsdl3.util.Processing;
import org.greenstone.gsdl3.gs3build.xpointer.XPointer;
import org.greenstone.gsdl3.gs3build.metadata.*;
import org.greenstone.gsdl3.gs3build.util.DOMUtils;

import java.io.InputStream;
import java.io.OutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.io.File;
import java.util.Iterator;

import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.Document;
import org.w3c.dom.NamedNodeMap;

import org.greenstone.gsdl3.util.XMLConverter;

public class MGPPIndexer //extends AbstractIndexer
    implements IndexerInterface
{ 
    int          pass;
    int          documentSeqNo;
    int          sectionSeqNo;
    boolean      firstDocument;
    File indexDirectory;
    //    File textDirectory;
    String       indexStem;
    //    String       textStem;
    StringBuffer indexBuffer;
    String       outputDirectory;
    String       overallName; 
    
    List indexes;
    MGPPIndex current_index = null;
    MGPPPassesWrapper mgppPasses;
    

    static final String DOCUMENT = "Doc";
    static final String SECTION = "Sec";
    static final String START_OF_DOCUMENT = "<"+DOCUMENT+">";
    static final String END_OF_DOCUMENT = "</"+DOCUMENT+">";
    static final String START_OF_SECTION = "<"+SECTION+">";
    static final String END_OF_SECTION = "</"+SECTION+">";
    
    
    public static final String MGPP_INDEX_TYPE = "mgpp";
    public static final String INDEX_FILE_STEM = "index";


    class MGPPIndex
    {
	public String name = null;
	public String doc_level = null; 
	public List levels = null;
	public List fields = null;
	boolean error = false;// assume built until we get an error
	
	public MGPPIndex(Element index_element) {
	    
	    this.fields = new ArrayList();
	    this.levels = new ArrayList();
	    this.name = index_element.getAttribute(GSXML.NAME_ATT);
	    if (this.name.equals("")) {
		// TODO make this dynamic
		this.name = "xx";
	    }
	    NodeList children = index_element.getChildNodes();
	    for (int c = 0; c < children.getLength(); c ++) {
		Node child = children.item(c);
		
		if (child.getNodeType() == Node.ELEMENT_NODE) {
		    String name = child.getNodeName();
		    
		    if (name.equals(GSXML.LEVEL_ELEM)) {
			String level = DOMUtils.getNodeChildText(children.item(c));
			this.levels.add(level);
		    }
		    else if (name.equals(GSXML.FIELD_ELEM)) {
			String fieldName = DOMUtils.getNodeChildText(children.item(c));
			this.fields.add(fieldName);
		    }
		}
	    }
	}
	
	public MGPPIndex(String name) {
	    this.name = name;
	    this.doc_level = DOCUMENT;
	    this.fields = new ArrayList();
	    this.levels = new ArrayList();
	}
	
	public void setDocLevel(String doc_level) {
	    this.doc_level = doc_level;
	}

	public void addLevel(String level) {
	    this.levels.add(level);
	}

	// change to allow nested fields
	public void addField(String field) {
	    this.fields.add(field);
	}
	public List getLevels() {
	    return this.levels;
	}
	public List getFields() {
	    return this.fields;
	}
	public String getName() {
	    return this.name;
	}
	public boolean hasError() {
	    return this.error;
	}
	public void setError(boolean b) {
	    this.error = b;
	}

    }
    
    
    public MGPPIndexer(String name)
    { 
	this.overallName = name;
	this.indexes = new ArrayList();
	//this.passExtra = "";
    }

    public String getName()
    { 
	return this.overallName;
    }
    
    public String getIndexType()
    { 
	return MGPP_INDEX_TYPE;
    }

    // for now make all indexes use document and section levels. 
    // then when writing the buildconfig, only display the levels 
    // that the user has specified (likely to be both doc and sec).
    public boolean configure(Node search_node)
    {
	NodeList index_children = GSXML.getChildrenByTagName(search_node, GSXML.INDEX_ELEM);
	
	// add a text 'index' - we should be able to turn this off in the config file? 
	MGPPIndex text_index = new MGPPIndex("text");
	text_index.addField("text");
	// always do eveything at doc and sec level at the moment
	text_index.addLevel(DOCUMENT);
	text_index.addLevel(SECTION);
	indexes.add(text_index);
	
	for (int i = 0; i < index_children.getLength(); i ++) {
	    Element index_elem = (Element)index_children.item(i);
	    MGPPIndex index = new MGPPIndex(index_elem);
	    if (index.getName() != null && index.getLevels() != null && index.getFields()!= null) {
		indexes.add(index);
	    } else {
		System.err.println("invalid index spec, not including"+new XMLConverter().getPrettyString(index_elem));
	    }
	}
	// TODO make sure all index names are unique
	return true;
    }

    /**
     *  The output directory should be (collection)/building/text/ for
     *  normal Greenstone builds
     */ 
    public boolean configure(String label, String value)
    { 
	if (label.equals(IndexerManager.outputDir)) {
	    this.outputDirectory = value;
	    
	    // attempt to ensure that the text subdirectory exists
// 	    this.textDirectory = new File(outputDirectory, "text");
// 	    if (!textDirectory.exists()) {
// 		if (!textDirectory.mkdir()) {
// 		    return false;
// 		}
// 	    }
// 	    else if (!textDirectory.isDirectory()) {
// 		return false;
// 	    }
// 	    this.textStem = this.textDirectory.getPath() + File.separator + INDEX_FILE_STEM;
	    
// 	    // attempt to ensure that the index subdir exists
// 	    this.indexDirectory = new File(outputDirectory, "idx");
// 	    if (!indexDirectory.exists()) {
// 		if (!indexDirectory.mkdir()) {
// 		    return false;
// 		}
// 	    }
// 	    else if (!indexDirectory.isDirectory()) {
// 		return false;
// 	    }
// 	    this.indexStem = this.indexDirectory.getPath() + File.separator + INDEX_FILE_STEM;
	    
	    // Sign to the user which mg directory is being used...
// 	    System.out.println("Output MGPP text directory is " + this.textStem);
// 	    System.out.println("Output MGPP index directory is " + this.indexStem); 
	}
	this.pass = 0;
	return true;
    }


//     public boolean addIndex(String name, String level, String field)
//     { 
// // 	if (level == "doc_level") {
// // 	    passExtra = " -J " + level;
// // 	}
// // 	else {
// // 	    passExtra = " -K " + level;
// // 	}
// 	return true;
//     }

    /**
     *  Index a single document; the document interface can be used to extract individual
     *  metadata items etc. as required or desired and index those instead or as well as
     *  the body text of the document.
     */
    public boolean indexDocument(DocumentID docID, DocumentInterface document)
    {
	if (this.pass == 0) {
	    document.removeAllMetadata("gsdl3", "mgseqno");
	    document.addDocumentMetadata("gsdl3", "mgseqno", this.overallName+"."+Integer.toString(this.sectionSeqNo));
	}
	
	
	String docText = null;
	
	this.sectionSeqNo ++;
	
	Document domDocument = document.getDOMDocument();
	if (domDocument != null) {
	    System.err.println("dom doc is not null");
	    METSStructure sections = document.getDocumentStructure().getStructure("Section");
	    if (sections != null) {
		System.err.println("sections are not null");
		docText = this.prepareDOM(document, domDocument, sections, "gsdl3"); 
	    }
	}
	if (docText == null) {
	    System.err.println("dom doc or sections was null - asking for doc text");
	    StringBuffer doc_text_buffer = new StringBuffer();
	    List fields = this.current_index.getFields();
	    for (int i=0; i<fields.size(); i++) {
		String field = (String)fields.get(i);
		if (field.equals("text")) {
		    doc_text_buffer.append(document.getDocumentText());
		    doc_text_buffer.append(" ");
		}  else {
		    // its a metadata - do namespace properly!!
		    List values = document.getDocumentMetadataItem("gsdl3", field);
		    if (values != null) {
			Iterator valueIter = values.iterator();
			while (valueIter.hasNext()) {
			    String value = valueIter.next().toString();
			    doc_text_buffer.append(value);
			    doc_text_buffer.append(" ");
			}
		    }
		}
	    } // for each field
	    docText = doc_text_buffer.toString();
	    sectionSeqNo ++;
	}
	
	this.indexBuffer.append(START_OF_DOCUMENT);
	this.indexBuffer.append(START_OF_SECTION);
	this.indexBuffer.append(docText);
	this.indexBuffer.append(END_OF_SECTION);
	this.indexBuffer.append(END_OF_DOCUMENT);
	this.mgppPasses.processDocument(indexBuffer.toString());
	this.indexBuffer.delete(0, this.indexBuffer.length());

	this.firstDocument = false;

	this.documentSeqNo++;
	
	return true;
    }

    /**
     *  Initialise the pass: open required files, check status
     */
    public boolean startPass(int passNumber)
    { 
	this.pass = passNumber;
	this.firstDocument = true;
	this.documentSeqNo = 1;
	this.sectionSeqNo = 1;

	this.mgppPasses = new MGPPPassesWrapper();
	this.indexBuffer = new StringBuffer();
	
	int indexNo = this.pass/2;
	this.current_index = (MGPPIndex) this.indexes.get(indexNo);

	if (this.current_index.hasError()) {
	    // an error has already occurred for this index, don't continue
	    System.out.println("pass "+this.pass+": aborted due to errors in the previous pass");
	    return false;
	}
	
	// attempt to ensure that the text/index subdirectory exists
	this.indexDirectory = new File(outputDirectory, current_index.getName());
	if (!indexDirectory.exists()) {
	    if (!indexDirectory.mkdir()) {
		return false;
	    }
	}
	else if (!indexDirectory.isDirectory()) {
	    return false;
	}

	this.indexStem = this.indexDirectory.getPath() + File.separatorChar + INDEX_FILE_STEM;  // TODO: modify for index
// 	if (this.pass == 0) {
// 	    // first pass, also set up the textStem
// 	    this.textDirectory = this.indexDirectory;
// 	    this.textStem = this.indexStem;
// 	}

	// get the parameters for this execution of mg_passes
	mgppPasses.setFileName(this.indexStem);
	if (!Misc.isWindows()) {
	    mgppPasses.setBasePath("/");
	}
	
	// always use Doc and Sec for now
	mgppPasses.setDocumentTag(DOCUMENT);
	mgppPasses.addLevelTag(SECTION);
	
	//this.currentIndexLevel = "Document";// index.getLevel();
	//this.currentIndexField = "text";//index.getField();
	//this.currentIndexName = "idx"; //index.getName();
	    
	int mgppPass = this.pass < 2 ? this.pass : ((this.pass % 2) + 2);
	switch (mgppPass) {
	case 0:
	    // -T1
	    mgppPasses.addPass(MGPPPassesWrapper.TEXT_PASS_1);
	    break;
	    
	case 1:
	    // -T2
	    mgppPasses.addPass(MGPPPassesWrapper.TEXT_PASS_2);
	    break;
	    
	case 2:
	    // -I1
	    mgppPasses.addPass(MGPPPassesWrapper.INDEX_PASS_1);
	    break;
	    
	case 3:
	    // -I2
	    mgppPasses.addPass(MGPPPassesWrapper.INDEX_PASS_2);
	    break;
	}
	
	mgppPasses.init();
	System.out.println("Pass " + this.pass);
	return true;
    }

    /**
     *  Complete a pass - reset file counters, close files, etc.
     */
    public boolean endPass(int passNumber)
    { 
	try {
	    this.indexBuffer.append(END_OF_DOCUMENT);
	    mgppPasses.processDocument(indexBuffer.toString());
	    this.indexBuffer.delete(0, this.indexBuffer.length());
	    Thread.sleep(1000); // what for??
	}
	catch (InterruptedException ex) {
	    System.out.println(ex);
	}
	
	mgppPasses.finish();
	
	try {
	    Thread.sleep(1000);
	} catch (Exception e) {}
	
	int exit_value = mgppPasses.exitValue();
	System.out.println("Pass " + this.pass + " completed with " + exit_value);
	if (exit_value !=0) {
	    this.current_index.setError(true);
	    return false;
	}
	
	int mgppPass = this.pass < 2 ? this.pass : ((this.pass % 2) + 2);
	
	String osextra = "";
	if (!Misc.isWindows()) {
	    osextra = " -d / ";
	}

	switch (mgppPass) {
	case 0:
	    System.out.println("Compressing dictionary");
	    exit_value = Processing.runProcess("mgpp_compression_dict -f " + this.indexStem + " -S -H -2 -k 5120"+ osextra);
	    
	    if (exit_value == 0) {
		System.out.println("Compressed dictionary successfully written");
	    } else {
		System.err.println("Error from mgpp_compression_dict: " + exit_value);
		this.current_index.setError(true);
		return false;
	    }
	    break;
	    
	case 2:
	    System.out.println("Creating perfect hash");
	    exit_value = Processing.runProcess("mgpp_perf_hash_build -f " + this.indexStem + osextra);
	    if (exit_value ==0) {
		System.out.println("Perfect hashes completed");
	    } else {
		System.err.println("Unable to build the perfect hash");
		this.current_index.setError(true);
		return false;
	    }
	    break;
	    
	case 3:
	    System.out.println("Writing weights file");
	    exit_value = Processing.runProcess("mgpp_weights_build -f " + this.indexStem + osextra);
	    if (exit_value ==0) {
		System.out.println("Weights file successfully written");
	    } else {
		System.err.println("Unable to create weights file");
		this.current_index.setError(true);
		return false;
	    }
	    
	    System.out.println("Creating inverted dictionary");
	    exit_value = Processing.runProcess("mgpp_invf_dict -f " + this.indexStem + osextra);
	    if (exit_value ==0) {
		System.out.println("Inverted dictionary file successfully written");
	    } else {
		System.out.println("Unable to create inverted dictionary file");
		this.current_index.setError(true);
		return false;
	    }
	    
	    System.out.println("Creating Stem indexes");
	    exit_value = Processing.runProcess("mgpp_stem_idx -b 4096 -s1 -f " + this.indexStem +osextra);
	    if (exit_value == 0) {
		System.out.println("Stemmed index 1 successfully written");
	    } else {
		System.out.println("Unable to create stemmed index 1");
		this.current_index.setError(true);
		return false;
	    }
	    
	    exit_value = Processing.runProcess("mgpp_stem_idx -b 4096 -s2 -f " + this.indexStem + osextra);
	    if (exit_value == 0) {
		System.out.println("Stemmed index 2 successfully written");
	    } else {
		System.out.println("Unable to create stemmed index 2");
		this.current_index.setError(true);
		return false;
	    }
	    exit_value = Processing.runProcess("mgpp_stem_idx -b 4096 -s3 -f " + this.indexStem + osextra);
	    if (exit_value == 0) {
		System.out.println("Stemmed index 3 successfully written");
	    } else {
		System.out.println("Unable to create stemmed index 3");
		this.current_index.setError(true);
		return false;
	    }
	    
	    break;
	} // switch
	
	return true;
    }
	
    /**
     *  Do any tidying up
     */
    public void tidyup()
    {
    }
    
    /**
     *  Return the number of passes required for this index.
     */
    public int getNumberOfPasses()
    { 
	//return this.indexes.size()*2;
	return 4;
    }

    public boolean addServiceDescriptions(Element service_rack_list) {
	
	// we only have one real index at the moment, - the first index in the list will be the text one.
	MGPPIndex index = (MGPPIndex)this.indexes.get(1);
	if (index.hasError()) {
	    // we weren't able to create any indexes - don't add a search service
	    return false;
	}
	
	Document doc = service_rack_list.getOwnerDocument();
	Element search_service_elem = doc.createElement(GSXML.SERVICE_CLASS_ELEM);
	service_rack_list.appendChild(search_service_elem);
	Element retrieve_service_elem = doc.createElement(GSXML.SERVICE_CLASS_ELEM);		
	service_rack_list.appendChild(retrieve_service_elem);
	
	// generate the list of indexes - with only one index in it
	Element index_list = doc.createElement(GSXML.INDEX_ELEM+GSXML.LIST_MODIFIER);
	Element index_elem = doc.createElement(GSXML.INDEX_ELEM);
	index_elem.setAttribute(GSXML.NAME_ATT, index.getName());
	index_list.appendChild(index_elem);
	
	Element default_index = doc.createElement("defaultIndex");
	default_index.setAttribute(GSXML.NAME_ATT, index.getName());

	Element base_index_name = doc.createElement("baseIndexPrefix");
	base_index_name.setAttribute(GSXML.NAME_ATT, overallName);

	Element index_stem = doc.createElement("indexStem");
	index_stem.setAttribute(GSXML.NAME_ATT, INDEX_FILE_STEM);
	
	Element default_level = doc.createElement("defaultLevel");
	default_level.setAttribute(GSXML.NAME_ATT, SECTION);

	// always have doc and sec at the moment
	Element level_list = doc.createElement(GSXML.LEVEL_ELEM+GSXML.LIST_MODIFIER);
	Element level = doc.createElement(GSXML.LEVEL_ELEM);
	level.setAttribute(GSXML.NAME_ATT, DOCUMENT);
	level_list.appendChild(level);

	level = doc.createElement(GSXML.LEVEL_ELEM);
	level.setAttribute(GSXML.NAME_ATT, SECTION);
	level_list.appendChild(level);
	
	Element field_list = doc.createElement(GSXML.FIELD_ELEM+GSXML.LIST_MODIFIER);
	Element field;
	List fields = index.getFields();
	for (int i=0; i<fields.size(); i++) {
	    String f = (String) fields.get(i);
	    field = doc.createElement(GSXML.FIELD_ELEM);
	    field.setAttribute(GSXML.NAME_ATT, f);
	    field_list.appendChild(field);
	}
	
	search_service_elem.setAttribute(GSXML.NAME_ATT, "GS3MGPPSearch");  
	search_service_elem.appendChild(index_list);
	search_service_elem.appendChild(default_index);
	search_service_elem.appendChild(level_list);
	search_service_elem.appendChild(default_level);
	search_service_elem.appendChild(field_list); 
	search_service_elem.appendChild(base_index_name);
	search_service_elem.appendChild(index_stem);
	
	retrieve_service_elem.setAttribute(GSXML.NAME_ATT, "GS3MGPPRetrieve");
	retrieve_service_elem.appendChild(default_level.cloneNode(true));
	retrieve_service_elem.appendChild(base_index_name.cloneNode(true));
	retrieve_service_elem.appendChild(index_stem.cloneNode(true));

	return true;
    }
 
 
    private Node recurseDOM(DocumentInterface metsDoc, Node node, 
			    AbstractStructure structure, StringBuffer textBuffer,
			    StringBuffer extraBuffer, String namespace) 
	//String name, String namespace, String field)
    {
	List fields = current_index.getFields();
	// send out the ctrl-c...if this is
	if (structure.getStructureType().equals(METSDivision.DIVISION_TYPE)) {
	    // try doing this for all index types
	    // actually we should only need to do this once ????
	    if (this.pass == 0) {
	    System.err.println("division structure");
	    //if ((this.currentIndexName != null)) { // && this.level != null && this.level.equals(IndexerInterface.SECTION_LEVEL)) { //name.startsWith("s")) {
		METSDivision division = (METSDivision) structure;

		// get the division metadata block
		METSDescriptive descriptive;
		String metadataId = division.getDefaultMetadataReference();
		if (metadataId == null) {
		    descriptive = metsDoc.getDocumentMetadata().createDescriptive(division.getLabel());
		    division.addMetadataReference(descriptive.getID());
		}
		else {
		    // Get the descriptive item...
		    descriptive = metsDoc.getDocumentMetadata().getDescriptiveById(metadataId);
		}
	    
		descriptive.addMetadata("gsdl3", "mgseqno", this.overallName + "." + Integer.toString(this.sectionSeqNo));
	    
		metsDoc.setChanged(true);
		//metsDoc.setModified(true);
		//	System.out.println("Assigning " + this.sectionSeqNo + " to " + metsDoc.getID() + " " + division.getLabel());
	    } // first pass
	
	    // append an 'end of section' marker
	    //textBuffer.append(END_OF_SECTION);
	    this.sectionSeqNo ++;
	    textBuffer.append(END_OF_SECTION);
	    textBuffer.append(START_OF_SECTION);
	    // for document-level indexes, always append an 'end of document' tag at the
	    // end of the document for each section.  Otherwise, each section is followed
	    // by an end of document character.  This ensures that all indexes use the
	    // same document numbering...
	    
// 	    if (this.current_index.getLevel().equals(IndexerInterface.DOCUMENT_LEVEL)) {
// 		extraBuffer.append(END_OF_DOCUMENT);
// 	    }
// 	    else {
// 		textBuffer.append(END_OF_DOCUMENT);
// 		this.documentSeqNo ++;
// 	    }
	    
	    // produce the body here for metadata output of divisions - in the case of
	    // text output, that will happen below...
	    
	    if (fields.size()>1 || !((String)fields.get(0)).equals("text")) {
		// if there is only text, don't do this
		METSDescriptive descriptive;
	
		METSDivision division = (METSDivision) structure;

		String metadataId = division.getDefaultMetadataReference();
		// are there other metadata refs to get??
		descriptive = metsDoc.getDocumentMetadata().getDescriptiveById(metadataId);
		if (descriptive != null) {
		    for (int i=0; i<fields.size(); i++) {
			String field = (String)fields.get(i);
			if (field.equals("text")) {
			    continue;
			}
			List values = descriptive.getMetadata(namespace, field);
			if (values != null) {	
			    Iterator valueIter = values.iterator();
			    while (valueIter.hasNext()) {
				String value = valueIter.next().toString();
				textBuffer.append("<"+field+">");
				textBuffer.append(value);
				textBuffer.append("</"+field+">");
			    }
			}
		    }
		}
	    }
	}
	
	// go through our children as required...
	Iterator children = structure.getChildIterator();
	Node startNode;
	boolean index_text = fields.contains("text");
	while (children.hasNext()) {
	    AbstractStructure child = (AbstractStructure) children.next();
      
	    // get xpointer for child 
	    // get start position node
	    if (metsDoc.getDocumentType() == "METS"){
		startNode = ((METSDocument) metsDoc).getSectionStartNode((METSDivision) child);	  
	    } else {
		startNode = ((HTMLDocument) metsDoc).getSectionStartNode((METSDivision) child);
	    }
	    
	    // while this node isn't the child's start node, produce the 
	    // HTML node text, if in text field mode...
	    if (index_text) {
		while (node != startNode) {
		    XPointer.printNode(node, textBuffer, false);
		    node = XPointer.getNextNode(node);
		}
	    }
	    
	    // recurse to child
	    node = this.recurseDOM(metsDoc, node, child, textBuffer, extraBuffer, namespace); // name, namespace, field);
	} // while next child

	// close a document - the actual closing \B will be done by the main
	// loop, so only a required \C is printed here...
	// why have we got STRUCTURE_TYPE here and DIVISION_TYPE above????
	if (structure.getStructureType().equals(METSStructure.STRUCTURE_TYPE)) {
	    if (this.pass == 0) {
		System.err.println("structure structure");
	    }
	    if (index_text) {
		while (node != null) {
		    XPointer.printNode(node, textBuffer, false);
		    node = XPointer.getNextNode(node);
		}
	    }
      
	    //textBuffer.append(END_OF_SECTION);
	    //this.sectionSeqNo ++;
      
	}
	return node;
    }

    
    private void printNode(Node node, StringBuffer buffer, 
			   boolean preserve_format, boolean close) {
	
	if (node.getNodeType() == org.w3c.dom.Node.TEXT_NODE) {
	    if (!close) {
		buffer.append(node.getNodeValue());
		buffer.append(" ");
	    }
	    return;
	}
	
	if (node.getNodeType() == org.w3c.dom.Node.ENTITY_NODE) {
	   if (!close) {
	       buffer.append("&");
	       buffer.append(node.getNodeValue());
	       buffer.append(";");
	       buffer.append(" ");
	   }
	   return;
	}
	if (!preserve_format) return; // we don't want any xml output
	/*
	else if (node.getNodeType() == org.w3c.dom.Node.COMMENT_NODE) {
      if (!close) {
	buffer.append("<!-- ");
	buffer.append(node.getNodeValue());
	buffer.append(" -->");
      }
      }*/
	if (node.getNodeType() == org.w3c.dom.Node.ELEMENT_NODE) {
	    
	    boolean hasChildren = (node.getChildNodes().getLength() > 0);
	    
	    if (close) {
		if (hasChildren) {
		    // put the close node
		    buffer.append("</");
		    buffer.append(node.getNodeName());
		    buffer.append(">");
		}
		// else it was closed off previously
		return;
	    }
	    
	    
	    buffer.append("<");
	    buffer.append(node.getNodeName());
	    NamedNodeMap attributes = ((Element) node).getAttributes();
	    for (int a = 0; a < attributes.getLength(); a ++) {
		Node attributeNode = attributes.item(a);
		buffer.append(" ");
		buffer.append(attributeNode.getNodeName());
		
		String value = attributeNode.getNodeValue();
		if (value != null && value.length() > 0) {
		    buffer.append("=\"");
		    buffer.append(value);
		    buffer.append("\"");
		}
	    }
	    

	    if (!hasChildren) {
		buffer.append(" /");
	    }
	    buffer.append(">");
	    
	    //if (close && node.getNodeName()=="Section") {
	    //buffer.append((char) 3);
	    // }
	}
	
    }
    
    /*
    private Node recurseDOMold(DocumentInterface metsDoc, Node node, 
			    AbstractStructure structure, StringBuffer textBuffer,
			    StringBuffer extraBuffer, String namespace) 
	//String name, String namespace, String field)
    {
	List fields = current_index.getFields();
	// send out the ctrl-c...if this is
	if (structure.getStructureType().equals(METSDivision.DIVISION_TYPE)) {
	// try doing this for all index types
	    if ((this.currentIndexName != null)) { // && this.level != null && this.level.equals(IndexerInterface.SECTION_LEVEL)) { //name.startsWith("s")) {
	    METSDivision division = (METSDivision) structure;

	    // get the division metadata block
	    METSDescriptive descriptive;
	    String metadataId = division.getDefaultMetadataReference();
	    if (metadataId == null) {
		descriptive = metsDoc.getDocumentMetadata().createDescriptive(division.getLabel());
		division.addMetadataReference(descriptive.getID());
	    }
	    else {
		// Get the descriptive item...
		descriptive = metsDoc.getDocumentMetadata().getDescriptiveById(metadataId);
	    }
	    
	    descriptive.addMetadata("gsdl3", "mgseqno", this.name + "." + Integer.toString(this.sectionSeqNo));
	    
	    metsDoc.setChanged(true);
	    //metsDoc.setModified(true);
	    //	System.out.println("Assigning " + this.sectionSeqNo + " to " + metsDoc.getID() + " " + division.getLabel());
	} // section level
	
	// append an 'end of section' marker
	//textBuffer.append(END_OF_SECTION);
	this.sectionSeqNo ++;
	
	// for document-level indexes, always append an 'end of document' tag at the
	// end of the document for each section.  Otherwise, each section is followed
	// by an end of document character.  This ensures that all indexes use the
	// same document numbering...
	if (this.currentIndexLevel == null ||
	    this.currentIndexLevel.equals(IndexerInterface.DOCUMENT_LEVEL)) {
	    extraBuffer.append(END_OF_DOCUMENT);
	}
	else {
	    textBuffer.append(END_OF_DOCUMENT);
	    this.documentSeqNo ++;
	}
	
	// produce the body here for metadata output of divisions - in the case of
      // text output, that will happen below...
      if (!this.currentIndexField.equals("text")) 
      { METSDescriptive descriptive;
	
	METSDivision division = (METSDivision) structure;

	String metadataId = division.getDefaultMetadataReference();
	    
	descriptive = metsDoc.getDocumentMetadata().getDescriptiveById(metadataId);
	if (descriptive != null) {
	  List values = descriptive.getMetadata(namespace, this.currentIndexField);
	  
	  if (values != null) {	
	    Iterator valueIter = values.iterator();
	    while (valueIter.hasNext()) {
	      String value = valueIter.next().toString();
	      
	      textBuffer.append(value);
	      if (valueIter.hasNext()) {
		  //textBuffer.append(END_OF_SECTION);
	      }
	    }
	  }
	}
      }
    }

    // go through our children as required...
    Iterator children = structure.getChildIterator();
    Node startNode;
    while (children.hasNext()) {
      AbstractStructure child = (AbstractStructure) children.next();
      
      // get xpointer for child 
      // get start position node
      if (metsDoc.getDocumentType() == "METS"){
	  startNode = ((METSDocument) metsDoc).getSectionStartNode((METSDivision) child);	  
      } else {
	  startNode = ((HTMLDocument) metsDoc).getSectionStartNode((METSDivision) child);
      }
	    //Node startNode = ((HTMLDocument) metsDoc).getSectionStartNode((METSDivision) child);
      
      // while this node isn't the child's start node, produce the HTML node text, if
      // in text field mode...
      if (this.currentIndexField.equals("text")) {
	while (node != startNode) {
	  XPointer.printNode(node, textBuffer, false);

	  // print buffer to node
	  node = XPointer.getNextNode(node, (this.currentIndexField.equals("text") ? textBuffer : null));
	}
      }
      
      // recurse to child
      node = this.recurseDOM(metsDoc, node, child, textBuffer, extraBuffer, namespace); // name, namespace, field);
    } // while next child

    // close a document - the actual closing \B will be done by the main
    // loop, so only a required \C is printed here...
    if (structure.getStructureType().equals(METSStructure.STRUCTURE_TYPE)) {
      while (node != null) {
	if (this.currentIndexField.equals("text")) {
	  XPointer.printNode(node, textBuffer, false);
	}
	node = XPointer.getNextNode(node, (this.currentIndexField.equals("text") ? textBuffer : null));
      }
      
      //textBuffer.append(END_OF_SECTION);
      this.sectionSeqNo ++;
      
    }
    return node;
  }
    */
    private String prepareDOM(DocumentInterface metsDoc, Document document, METSStructure structure, String namespace)
	//  String name, String namespace, String field)
    { 
	StringBuffer extraBuffer = new StringBuffer();
	Node node = document.getDocumentElement();
	StringBuffer textBuffer = new StringBuffer();
	
	this.recurseDOM(metsDoc, node, structure, textBuffer, extraBuffer, namespace); 
	textBuffer.append(extraBuffer.toString());
	return textBuffer.toString();
    }
    
}
