package org.greenstone.gsdl3.util;


import org.xml.sax.InputSource;
import org.xml.sax.EntityResolver;
import java.io.File;
import java.net.URL;

// uses a class loader to find entities
// The class loader to use can be set by setClassLoader(), otherwise it will use the class loader that loaded itself. For the Tomcat webapp, this will be the webapp class loader, not the system class loader. the webapp classloader knows about the classes in the WEB-INF/classes dir, the system classloader knows about the ones on the classpath. The system class loader is a parent to web app classloader, so this will be used as well.

public class GSEntityResolver implements EntityResolver {

    ClassLoader class_loader = null;
    
    public void setClassLoader(ClassLoader loader) {
	this.class_loader = loader;
    }
    
    public InputSource resolveEntity (String public_id, String system_id) {
	
	//System.out.println("entity resolver for "+system_id);
	String temp_id = system_id;
	if (temp_id.startsWith("file://")) {
	    File f = new File(system_id);
	    if (f.exists()) {
		return new InputSource(system_id);
	    } else {
		temp_id = temp_id.substring(temp_id.lastIndexOf("/")+1);
	    }
	} else {
	    if (temp_id.indexOf("/")!= -1) {
		temp_id = temp_id.substring(temp_id.lastIndexOf("/")+1);
	    }
	}
	// try using a class loader
	if (this.class_loader==null) {
	    this.class_loader = this.getClass().getClassLoader();
	}
	URL url = class_loader.getResource(temp_id);
	if (url == null) {
	    return null;
	}
	return new InputSource("file://"+url.getFile());
    }
}
