<?xml version="1.0" encoding="ISO-8859-1"?>
<xsl:stylesheet version="1.0"
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:java="http://xml.apache.org/xslt/java"
  xmlns:util="xalan://org.greenstone.gsdl3.util.XSLTUtil"
  extension-element-prefixes="java util"
  exclude-result-prefixes="java util">

  <!-- some global parameters - these are set by whoever is invoking the transformation -->
  <xsl:param name="interface_name"/>
  <xsl:param name="library_name"/>
   <!-- global style info goes here  -->
  <xsl:template name="globalStyle">
    <link rel="stylesheet" href="interfaces/default/style/core.css" type="text/css"/>
  </xsl:template>
  
  <!-- the main page layout template is here -->
  <xsl:template match="page">
    <html>
     	 <xsl:call-template name="pageHead"/>
	 
   <body>
	<table border="0" cellspacing="0" cellpadding="0" width="100%">
	  <td valign="top" width="65"><!-- a space for our side bar-->
	    <img src="interfaces/default/images/spacer.gif" width="65" height="1" alt="" border="0"/><!-- an image cos netscape ignores the cell width -->
	  </td>
	  <td>
	    <table width="100%">
	      <tr><td>
		  <xsl:apply-templates select="pageResponse"/>
		  <xsl:if test="descendant::error">
		    <script language="Javascript">
		      <xsl:text disable-output-escaping="yes">
			function removeAllChildren(node) {
			 while (node.hasChildNodes()) {
			  node.removeChild(node.firstChild);
			 }
			}

		      function toggleHideError(obj) {
		      if (obj.style.display == "none") {
			obj.style.display = "";
			hide_link = document.getElementById("hide");
			removeAllChildren(hide_link);
			hide_link.appendChild(document.createTextNode("Hide Error"));
		      } else {
			obj.style.display = "none";
			hide_link = document.getElementById("hide");
			removeAllChildren(hide_link);
			hide_link.appendChild(document.createTextNode("Show Error"));
			
			}
		      }
		      </xsl:text>
		    </script>
		    <p align='right'><a id="hide" href="javascript:toggleHideError(error);">Show Error</a></p>
		    <div id="error" style="display: none;">
		      <xsl:apply-templates select="descendant::error"/>
		    </div>
		  </xsl:if>
		  <xsl:call-template name="greenstoneFooter"/>
		</td></tr>
	    </table>
	  </td>
	</table>
      </body>
    </html>
  </xsl:template>

  <!-- put a space in the title in case the actual value is missing - mozilla will not display a page with no title-->
  <xsl:template name="pageHead">
    <head>
      <title>
	<xsl:call-template name="pageTitle"/><xsl:text> </xsl:text>
      </title>
      <xsl:call-template name="globalStyle"/>
      <xsl:call-template name="pageStyle"/>
    </head>
  </xsl:template>


  <xsl:template name="greenstonePageBanner">
    <center><img src="interfaces/default/images/gsdlhead.gif"><xsl:attribute name="alt"><xsl:value-of select="util:getInterfaceText($interface_name, /page/@lang, 'gsdl')"/></xsl:attribute></img></center>
    <xsl:call-template name="dividerBar"/>
  </xsl:template>

  <xsl:template name="greenstoneFooter">
    <center>
      <p/>
      <small><xsl:value-of select="util:getInterfaceText($interface_name, /page/@lang, 'gs3power')"/></small>
    </center>
  </xsl:template>
  
  <xsl:template match="error">
    Error: <xsl:value-of select="."/>
  </xsl:template>

  <xsl:template name="standardPageBanner">
    <xsl:param name="collName"/>
    <xsl:param name="pageType"/>
    <xsl:variable name="this-element" select="/page/pageResponse/collection|/page/pageResponse/serviceCluster"/>
    <table width="100%">
      <tr>
	<td align="left">
	  <a href="{$library_name}?a=p&amp;sa=about&amp;c={$collName}">
	    <xsl:choose>
	      <xsl:when test="$this-element/displayItem[@name='icon']">
		<img border="0">
		  <xsl:attribute name="src">
		    <xsl:value-of select="$this-element/metadataList/metadata[@name='httpPath']"/>/images/<xsl:value-of select="$this-element/displayItem[@name='icon']"/>
		  </xsl:attribute>	
		  <xsl:attribute name="alt">
		    <xsl:value-of select="$this-element/displayItem[@name='name']"/>
		  </xsl:attribute>
		  <xsl:attribute name="title">
		    <xsl:value-of select="util:getInterfaceText($interface_name, /page/@lang, 'aboutpage')"/>
		  </xsl:attribute>
		</img>
	      </xsl:when>
	      <xsl:otherwise>
		<b><xsl:value-of select="$this-element/displayItem[@name='name']"/></b>
	      </xsl:otherwise>
	    </xsl:choose>
	  </a>
	</td>
	<td align="right">
	  <xsl:call-template name="top-buttons">
	    <xsl:with-param name="collName" select="$collName"/>
	    <xsl:with-param name="pageType" select="$pageType"/>
	  </xsl:call-template>
	</td>
      </tr>
    </table>
    <p/>
  </xsl:template>
  
  <xsl:template name="top-buttons">
    <xsl:param name="collName"/>
    <xsl:param name="pageType"/>
    <table cellspacing='4'>
      <tr>
	<td class="gsbutton">
	  <a class="gsbutton" href="{$library_name}?a=p&amp;sa=home"><xsl:attribute name="title"><xsl:value-of select="util:getInterfaceText($interface_name, /page/@lang, 'home_tip')"/></xsl:attribute>
	    <xsl:value-of select="util:getInterfaceText($interface_name, /page/@lang, 'home_b')"/>
	  </a>
	</td>
	<xsl:choose>
	  <xsl:when test="$pageType='help'">
	    <td class="gsbuttonoff">
	      <xsl:value-of select="util:getInterfaceText($interface_name, /page/@lang, 'help_b')"/>
	    </td>
	  </xsl:when>
	  <xsl:otherwise>
	    <td class="gsbutton">
	      <a class="gsbutton" href="{$library_name}?a=p&amp;sa=help&amp;c={$collName}"><xsl:attribute name="title"><xsl:value-of select="util:getInterfaceText($interface_name, /page/@lang, 'help_tip')"/></xsl:attribute>
		<xsl:value-of select="util:getInterfaceText($interface_name, /page/@lang, 'help_b')"/>
	      </a>
	    </td>
	  </xsl:otherwise>
	</xsl:choose>
	<xsl:choose>
	  <xsl:when test="$pageType='pref'">
	    <td class="gsbuttonoff">
	      <xsl:value-of select="util:getInterfaceText($interface_name, /page/@lang, 'pref_b')"/>
	    </td>
	  </xsl:when>
	  <xsl:otherwise>
	    <td class="gsbutton">
	      <a class="gsbutton" href="{$library_name}?a=p&amp;sa=pref&amp;c={$collName}"><xsl:attribute name="title"><xsl:value-of select="util:getInterfaceText($interface_name, /page/@lang, 'pref_tip')"/></xsl:attribute>
		<xsl:value-of select="util:getInterfaceText($interface_name, /page/@lang, 'pref_b')"/>
	      </a>
	    </td>
	  </xsl:otherwise>
	</xsl:choose>
      </tr>
    </table>
    
  </xsl:template>

  <xsl:template name="navigationBar">
    <xsl:param name="collName"/>
    <xsl:variable name="this-element" select="/page/pageResponse/collection|/page/pageResponse/serviceCluster"/>
    <xsl:variable name="this-service" select="/page/pageResponse/service/@name"/>
    <xsl:choose>
      <xsl:when test="$this-element/serviceList/service">
	<table border='0' cellspacing='0' width='100%'>
	  <tr>
	    <xsl:for-each select="$this-element/serviceList/service">
	      <xsl:variable name="action"><xsl:choose>
		  <xsl:when test="@name=$this-service">CURRENT</xsl:when>
		  <xsl:when test="@type='query'">q</xsl:when>
		  <xsl:when test="@type='browse'">b</xsl:when>
		  <xsl:when test="@type='process'">pr</xsl:when>
		  <xsl:when test="@type='applet'">a</xsl:when>
		  <xsl:otherwise>DO_NOT_DISPLAY</xsl:otherwise>
		</xsl:choose></xsl:variable>
	      <xsl:choose>
		<xsl:when test="$action='CURRENT'">
		  <td class="navbaroff"><nobr><xsl:value-of select="displayItem[@name='name']"/></nobr></td>
		</xsl:when>
		<xsl:when test="$action !='DO_NOT_DISPLAY'">
		  <td class="navbar"><a class="navbar" href="{$library_name}?a={$action}&amp;rt=d&amp;s={@name}&amp;c={$collName}"><xsl:if test="displayItem[@name='description']"><xsl:attribute name='title'><xsl:value-of select="displayItem[@name='description']"/></xsl:attribute></xsl:if><nobr><xsl:value-of select="displayItem[@name='name']"/></nobr></a></td>
		</xsl:when>
	      </xsl:choose>
	    </xsl:for-each>
	  </tr>
	</table>
      </xsl:when>
      <xsl:otherwise>
	<table border='0' cellspacing='0' width='100%'><tr><td class="navbar">&#160;</td></tr></table>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  
  
  <xsl:template name="dividerBar">
    <xsl:param name='text'/>
    <br/><table width='100%' border="0" cellpadding="0" cellspacing="0">
      <tr><td valign='bottom' height='20' background='interfaces/default/images/greenbar.gif'><font size="+1" color="black"><b>&#160;<xsl:value-of select='$text'/></b></font></td></tr></table><br/>
  </xsl:template>

  <!-- Display the appropriate image, depending on the node type -->
  <xsl:template match="documentNode" mode="displayNodeIcon">
    
    <!-- Root node: book icon (open or closed) -->
    <xsl:choose>
      <xsl:when test="@nodeType='root'">
	<xsl:choose>
	  <xsl:when test="documentNode">
	    <xsl:call-template name="openbookimg">
	      <xsl:with-param name="title">Close this book</xsl:with-param>
	    </xsl:call-template>
	  </xsl:when>
	  <xsl:otherwise>
	    <xsl:call-template name="closedbookimg">
	      <xsl:with-param name="title">Open this document and view contents</xsl:with-param>
	    </xsl:call-template>
	  </xsl:otherwise>
	</xsl:choose>
      </xsl:when>
      
      <!-- Internal node: folder icon (open or closed) -->
      <xsl:when test="@nodeType='internal'">
	<xsl:choose>
	  <xsl:when test="documentNode">
	    <xsl:call-template name="openfolderimg">
	      <xsl:with-param name="title">Close this folder</xsl:with-param>
	    </xsl:call-template>
	  </xsl:when>
	  <xsl:otherwise>
	    <xsl:call-template name="closedfolderimg">
	      <xsl:with-param name="title">Open this folder and view contents</xsl:with-param>
	    </xsl:call-template>
	  </xsl:otherwise>
	</xsl:choose>
      </xsl:when>
      
      <!-- Leaf node: page icon, and this is the default -->
      <xsl:otherwise>
	<xsl:call-template name="textpageimg">
	  <xsl:with-param name="title">View the document</xsl:with-param>
	</xsl:call-template>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template name="openbookimg">
    <xsl:param name="alt"/>
    <xsl:param name="title"/>
    <img border="0" width="28" height="23"
      src="interfaces/default/images/openbook.gif"
      alt="{$alt}" title="{$title}"/>
  </xsl:template>

  <xsl:template name="closedbookimg">
    <xsl:param name="alt"/>
    <xsl:param name="title"/>
    <img border="0" width="18" height="11"
      src="interfaces/default/images/book.gif"
      alt="{$alt}" title="{$title}"/>
  </xsl:template>

  <xsl:template name="openfolderimg">
    <xsl:param name="alt"/>
    <xsl:param name="title"/>
    <img border="0" width="23" height="15"
      src="interfaces/default/images/openfldr.gif"
      alt="{$alt}" title="{$title}"/>
  </xsl:template>

  <xsl:template name="closedfolderimg">
    <xsl:param name="alt"/>
    <xsl:param name="title"/>
    <img border="0" width="23" height="15"
      src="interfaces/default/images/clsdfldr.gif"
      alt="{$alt}" title="{$title}"/>
  </xsl:template>

  <xsl:template name="textpageimg">
    <xsl:param name="alt"/>
    <xsl:param name="title"/>
    <img border="0" width="16" height="21"
      src="interfaces/default/images/itext.gif"
      alt="{$alt}" title="{$title}"/>
  </xsl:template>

  <xsl:template name="bookshelfimg">
    <xsl:param name="alt"/>
    <xsl:param name="title"/>
    <img border="0" width="20" height="16"
      src="interfaces/default/images/bshelf.gif"  
      alt="{$alt}" title="{$title}"/>
  </xsl:template>
</xsl:stylesheet>  


